import asyncio
import logging
from typing import Optional

from channels.consumer import AsyncConsumer
from daphne.utils import import_by_path
from discord import utils
from discord.ext.commands import Bot
from discord.utils import MISSING
from django.conf import settings


class BaseDiscordConsumer(AsyncConsumer):
    bot_task: asyncio.Task = None
    bot: Bot = None

    async def start_discord_bot(self, event):
        print("Received start signal")
        if await self.is_bot_running():
            print("Bot is already running, doing nothing")
            return

        print("Unpacking signal for bot parameters")
        bot_path = event['bot_path']
        reconnect: bool = event['reconnect']

        log_handler: Optional[logging.Handler] = event.get('log_handler', MISSING)
        log_formatter: logging.Formatter = event.get('log_formatter', MISSING)
        log_level: int = event.get('log_level', MISSING)
        root_logger: bool = event.get('root_logger', False)

        print(f"Importing '{bot_path}' ...")
        self.bot: Bot = import_by_path(bot_path)

        print("Grabbing bot auth token from settings")
        bot_token: str = settings.DISCORD_PY_BOT_TOKEN

        async def runner(bot, token, reconnect):
            async with bot:
                await bot.start(token, reconnect=reconnect)

        if log_handler is not None:
            utils.setup_logging(
                handler=log_handler,
                formatter=log_formatter,
                level=log_level,
                root=root_logger,
            )

        print("Creating bot loop")
        bot_coro = runner(self.bot, bot_token, reconnect)
        print("Running bot")
        self.bot_task = asyncio.create_task(bot_coro)

    async def is_bot_running(self):
        if not self.bot_task:
            return False

        return not self.bot_task.done() and not self.bot_task.cancelled()

    async def send_message(self, event: dict):
        if not self.is_bot_running():
            raise ValueError("Bot is not running!")
        event.pop('type')
        channel: int = event.pop('channel')
        content: str = event.pop('content', None)

        await self.bot.get_channel(channel).send(
            content=content,
            **event,
        )



import logging
from typing import Any

import discord
from discord.ext import commands
from discord.ext.commands import Context
from django.conf import settings

description = """
A bot for friendly betting on league of legends
"""

intents = discord.Intents.default()
intents.message_content = True

bot = commands.Bot(command_prefix=settings.DISCORD_PY_COMMAND_PREFEX, description=description, intents=intents)

log = logging.getLogger(__name__)


@bot.event
async def on_command_error(ctx: Context, error):
    log.error("Discord bot had an error!", extra={'data': {'ctx': ctx, 'error': error}})
    await ctx.send("Something went wrong!")


@bot.event
async def on_error(event_method: str, /, *args: Any, **kwargs: Any):
    log.error("Discord bot had an error!", extra={'data': {'event_method': event_method, 'args': args, 'kwargs': kwargs}})


@bot.event
async def on_ready():
    print(f"Logged in as {bot.user} (ID: {bot.user.id})")
    print("------")


@bot.command(description="Say 'Hello, world!'")
async def hi(ctx: Context):
    print("Someone said 'Hi', responding!")
    await ctx.send(
        f"Hello, channel #{ctx.channel.id}!"
    )
